// Copyright 2017 Orus.io. All rights reserved.

package cmd

import (
	"net/http"
	"os"
	"path"

	"github.com/spf13/cobra"

	"bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/go-xbus/api"
	xbushttp "bitbucket.org/orus-io/xbus/http"
)

// serveCmd represents the "serve" command.
var serveCmd = &cobra.Command{
	Use:   "serve",
	Short: "Serve http server.",
	Run: func(cmd *cobra.Command, args []string) {
		os.Exit(serve())
	},
}

func serve() int {
	configDir := path.Dir(v.ConfigFileUsed())
	if _, err := xbus.LoadTLSOptionFiles(v, v.ConfigFileUsed(), Logger); err != nil {
		Logger.Errorf("%s", err)
		return 1
	}

	opts := xbus.COptionsFromViper(v, api.Account_GATEWAY, configDir, Logger)

	g := xbushttp.NewGateway(opts, Logger)
	if err := g.Startup(); err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	defer g.Shutdown()
	if err := http.ListenAndServe(":8911", g.Router); err != nil {
		Logger.Errorf("%s", err)
		return 1
	}
	return 0
}

func init() {
	RootCmd.AddCommand(serveCmd)
}
