package fullstack

import (
	"fmt"
	"testing"
	"time"

	xbus "bitbucket.org/orus-io/go-xbus"
	"bitbucket.org/orus-io/go-xbus/api"
	"github.com/stretchr/testify/assert"
)

func TestDirectorInvalidLogCrash(t *testing.T) {
	ft := InitFullstack(t, true)
	if ft == nil {
		return
	}
	defer ft.Shutdown()

	ft.CreateRole(api.Actor_EMITTER, "anyemitter")
	_, emitter := ft.CreateEmitter("bob", []string{"anyemitter"})

	assert.Equal(t, "bob", emitter.Name)

	ft.CreateRole(api.Actor_CONSUMER, "anyconsumer")

	_, consumer := ft.CreateConsumer("jojo", []string{"anyconsumer"})
	xsub, err := consumer.Subscribe(
		func(apc *xbus.ActorProcessingContext) error {
			return fmt.Errorf("test\000")
		})
	assert.Nil(t, err)
	defer xsub.Unsubscribe()

	assert.Equal(t, "jojo", consumer.Name)

	ft.InstallGraph(BuildSimpleGraph("evt1", "anyconsumer"))

	env := xbus.ForgeEnvelopeFromItems("evt1", []api.Item{[]byte("\"item1\"")})

	envStateChan, sub := ft.GetEnvStateChan(emitter, api.UUID{})
	defer sub.Unsubscribe()

	status, err := emitter.SendEnvelope(nil, api.DefaultOutput, env)
	assert.Nil(t, err)
	assert.Equal(t, api.EnvelopeAck_ACCEPTED, status)

	timeout := time.After(5 * time.Second)
	select {
	case <-timeout:
		t.Errorf("Process state not updated")
	case envState := <-envStateChan:
		assert.Equal(t, api.Process_RUNNING, envState.Status)
	}

	timeout = time.After(5 * time.Second)
	select {
	case <-timeout:
		t.Errorf("Process state not updated to 'error'")
	case envState := <-envStateChan:
		assert.Equal(t, api.Process_ERROR.String(), envState.Status.String())
	}
}
