package fullstack

import (
	"context"
	"testing"
	"time"

	"github.com/stretchr/testify/assert"

	"bitbucket.org/orus-io/go-xbus/api"
	test_storage "bitbucket.org/orus-io/xbus/testutils/storage"
)

func TestEnvelopeStorageService(t *testing.T) {
	ft := InitFullstack(t, true)
	if ft == nil {
		return
	}
	defer ft.Shutdown()

	envelopeStorage := ft.Server.GetEnvelopeStorage()

	fragments := test_storage.GenBigEnvFragments()
	for _, fragment := range fragments {
		_, err := envelopeStorage.StoreEnvelope(fragment)
		if err != nil {
			t.Fatal(err)
		}
	}
	lastFragment := fragments[len(fragments)-1]

	er := ft.Client.RetrieveEnvelope(
		context.Background(), "service", "-", lastFragment.ID,
	)

	envelope, err := er.Complete(4 * time.Second)
	if err != nil {
		t.Fatal(err)
	}
	for _, event := range envelope.Events {
		if event.Type == "xbus.test" {
			assert.Equal(t, api.Checksum(0xbdac9720), event.Checksum)
		} else if event.Type == "xbus.test.2" {
			assert.Equal(t, api.Checksum(0x560279ee), event.Checksum)
		} else {
			t.Fatal("Unexpected event in the envelope")
		}
	}
}
