// Copyright 2019 Orus.io. All rights reserved.

package storage

import (
	"testing"

	"github.com/stretchr/testify/assert"

	"bitbucket.org/orus-io/go-xbus/api"
	"bitbucket.org/orus-io/xbus/storage"
)

// PipelineStorageFactory is any factory of PipelineStorage
type PipelineStorageFactory func(bt testing.TB) (storage.PipelineStorage, func())

// PipelineStorageTests is a test suite for PipelineStorage implementations
func PipelineStorageTests(t *testing.T, factory PipelineStorageFactory) {
	t.Run("Save", func(t *testing.T) {
		ps, clean := factory(t)
		defer clean()

		id, err := ps.Save(storage.Pipeline{
			PipelineInfo: api.PipelineInfo{
				Name:    "test",
				Version: "0.1",
				Status:  api.PipelineInfo_DRAFT,
			},
			Graph: storage.Graph{
				Nodes: []storage.Node{{
					ID: "source",
				}},
			},
		})
		assert.Nil(t, err)

		p, ok := ps.Load(id)
		assert.True(t, ok)
		assert.Equal(t, "test", p.Name)
		assert.Equal(t, api.PipelineInfo_DRAFT, p.Status)
		assert.Equal(t, id, p.ID)

		ps.SetStatus(id, api.PipelineInfo_ACTIVE)

		p, ok = ps.Load(id)
		assert.True(t, ok)
		assert.Equal(t, api.PipelineInfo_ACTIVE, p.Status)

		_, err = ps.Save(p)
		assert.Equal(t, storage.ErrInvalidPipelineStatus, err)

		p.ID = api.UUID{}
		_, err = ps.Save(p)
		assert.Equal(t, storage.ErrInvalidPipelineStatus, err)
	})
}
