package xbus

import (
	"fmt"

	raven "github.com/getsentry/raven-go"
)

// RavenLogger reports all errors to sentry url and discard the rest
type RavenLogger struct {
	client     *raven.Client
	nextLogger Logger
}

// NewRavenLogger creates a new RavenLogger
func NewRavenLogger(client *raven.Client, nextLogger Logger) *RavenLogger {
	return &RavenLogger{
		client:     client,
		nextLogger: nextLogger,
	}
}

// Noticef logs a notice statement
func (r *RavenLogger) Noticef(format string, v ...interface{}) {
	r.nextLogger.Noticef(format, v...)
}

// Errorf logs an error statement
func (r *RavenLogger) Errorf(format string, v ...interface{}) {
	r.nextLogger.Errorf(format, v...)

	message := fmt.Sprintf(format, v...)
	packet := raven.NewPacket(
		message,
		&raven.Message{Message: format, Params: v},
	)

	eventID, ch := r.client.Capture(packet, map[string]string{})

	go func(eventId string, ch <-chan error, nextLogger Logger) {
		err := <-ch
		if err != nil {
			nextLogger.Errorf("Error sending %s to sentry: %s", eventID, err)
		}
	}(eventID, ch, r.nextLogger)
}

// Fatalf logs a fatal error
func (r *RavenLogger) Fatalf(format string, v ...interface{}) {
	r.nextLogger.Fatalf(format, v...)
}

// Debugf logs a debug statement
func (r *RavenLogger) Debugf(format string, v ...interface{}) {
	r.nextLogger.Debugf(format, v...)
}

// Tracef logs a trace statement
func (r *RavenLogger) Tracef(format string, v ...interface{}) {
	r.nextLogger.Tracef(format, v...)

}
