package version

//go:generate bash genversion.sh

import (
	"fmt"

	"github.com/spf13/cobra"
	flag "github.com/spf13/pflag"
)

// InitFlags add --version and --full-version to the flags
func InitFlags(flags *flag.FlagSet) {
	flags.Bool(
		"version", false,
		"Show version",
	)

	flags.Bool(
		"full-version", false,
		"Show version and commit hash",
	)
}

// Cmd implements default a cobra.Command 'Run' function that displays the
// version or usage depending on the flags
func Cmd(cmd *cobra.Command, args []string) {
	if cmd.Flag("version").Value.String() == "true" {
		fmt.Printf("%s %s\n", cmd.Use, Version)
	} else if cmd.Flag("full-version").Value.String() == "true" {
		build := ""
		if BuildNumber != "" {
			build = " build " + BuildNumber
		}
		fmt.Printf("%s %s %s%s\n", cmd.UseLine(), Version, CommitHash, build)
	} else {
		cmd.Usage()
	}
}
